@@ -49,7 +49,7 @@ import Data.Text (Text)
 import qualified Data.Text.Lazy as TL
 
 import System.Directory (doesFileExist, removeFile)
-import System.FilePath ((</>), takeDirectory)
+import System.FilePath  ( (</>) )
 
 import Agda.Benchmarking
 
@@ -97,11 +97,10 @@ import qualified Agda.Utils.Maybe.Strict as Strict
 import Agda.Utils.Monad
 import Agda.Utils.Null
 import Agda.Utils.IO.Binary
-import Agda.Utils.Pretty hiding (Mode)
+import Agda.Syntax.Common.Pretty hiding (Mode)
 import qualified Agda.Utils.ProfileOptions as Profile
 import Agda.Utils.Hash
 import qualified Agda.Utils.Trie as Trie
-import Agda.Utils.WithDefault
 
 import Agda.Utils.Impossible
 
@@ -124,24 +123,25 @@ data Source = Source
   , srcModule      :: C.Module              -- ^ The parsed module.
   , srcModuleName  :: TopLevelModuleName    -- ^ The top-level module name.
   , srcProjectLibs :: [AgdaLibFile]         -- ^ The .agda-lib file(s) of the project this file belongs to.
-  , srcCohesion    :: !CohesionAttributes
-    -- ^ Every encountered occurrence of a cohesion attribute.
+  , srcAttributes  :: !Attributes
+    -- ^ Every encountered attribute.
   }
 
 -- | Parses a source file and prepares the 'Source' record.
 
 parseSource :: SourceFile -> TCM Source
 parseSource sourceFile@(SourceFile f) = Bench.billTo [Bench.Parsing] $ do
-  (source, fileType, parsedMod, coh, parsedModName) <- mdo
+  (source, fileType, parsedMod, attrs, parsedModName) <- mdo
     -- This piece of code uses mdo because the top-level module name
     -- (parsedModName) is obtained from the parser's result, but it is
     -- also used by the parser.
     let rf = mkRangeFile f (Just parsedModName)
-    source                       <- runPM $ readFilePM rf
-    ((parsedMod, coh), fileType) <- runPM $ parseFile moduleParser rf $
-                                    TL.unpack source
-    parsedModName                <- moduleName f parsedMod
-    return (source, fileType, parsedMod, coh, parsedModName)
+    source                         <- runPM $ readFilePM rf
+    ((parsedMod, attrs), fileType) <- runPM $
+                                      parseFile moduleParser rf $
+                                      TL.unpack source
+    parsedModName                  <- moduleName f parsedMod
+    return (source, fileType, parsedMod, attrs, parsedModName)
   libs <- getAgdaLibFiles f parsedModName
   return Source
     { srcText        = source
@@ -150,7 +150,7 @@ parseSource sourceFile@(SourceFile f) = Bench.billTo [Bench.Parsing] $ do
     , srcModule      = parsedMod
     , srcModuleName  = parsedModName
     , srcProjectLibs = libs
-    , srcCohesion    = coh
+    , srcAttributes  = attrs
     }
 
 srcDefaultPragmas :: Source -> [OptionsPragma]
@@ -160,17 +160,19 @@ srcFilePragmas :: Source -> [OptionsPragma]
 srcFilePragmas src = pragmas
   where
   cpragmas = C.modPragmas (srcModule src)
-  pragmas = [ opts | C.OptionsPragma _ opts <- cpragmas ]
-
-srcPragmas :: Source -> [OptionsPragma]
-srcPragmas src = srcDefaultPragmas src ++ srcFilePragmas src
+  pragmas = [ OptionsPragma
+                { pragmaStrings = opts
+                , pragmaRange   = r
+                }
+            | C.OptionsPragma r opts <- cpragmas
+            ]
 
 -- | Set options from a 'Source' pragma, using the source
 --   ranges of the pragmas for error reporting.
 setOptionsFromSourcePragmas :: Source -> TCM ()
-setOptionsFromSourcePragmas src =
-  setCurrentRange (C.modPragmas . srcModule $ src) $
-    mapM_ setOptionsFromPragma (srcPragmas src)
+setOptionsFromSourcePragmas src = do
+  mapM_ setOptionsFromPragma (srcDefaultPragmas src)
+  mapM_ setOptionsFromPragma (srcFilePragmas    src)
 
 -- | Is the aim to type-check the top-level module, or only to
 -- scope-check it?
@@ -223,7 +225,7 @@ mergeInterface i = do
     reportSLn "import.iface.merge" 20 $
       "  Current builtins " ++ show (Map.keys bs) ++ "\n" ++
       "  New builtins     " ++ show (Map.keys bi)
-    let check b (Builtin x) (Builtin y)
+    let check (BuiltinName b) (Builtin x) (Builtin y)
               | x == y    = return ()
               | otherwise = typeError $ DuplicateBuiltinBinding b x y
         check _ (BuiltinRewriteRelations xs) (BuiltinRewriteRelations ys) = return ()
@@ -238,6 +240,8 @@ mergeInterface i = do
       (iUserWarnings i)
       (iPartialDefs i)
       warns
+      (iOpaqueBlocks i)
+      (iOpaqueNames i)
     reportSLn "import.iface.merge" 20 $
       "  Rebinding primitives " ++ show prim
     mapM_ rebind prim
@@ -247,7 +251,7 @@ mergeInterface i = do
     where
         rebind (x, q) = do
             PrimImpl _ pf <- lookupPrimitiveFunction x
-            stImportedBuiltins `modifyTCLens` Map.insert x (Prim pf{ primFunName = q })
+            stImportedBuiltins `modifyTCLens` Map.insert (someBuiltin x) (Prim pf{ primFunName = q })
 
 addImportedThings
   :: Signature
@@ -258,9 +262,11 @@ addImportedThings
   -> Map A.QName Text      -- ^ Imported user warnings
   -> Set QName             -- ^ Name of imported definitions which are partial
   -> [TCWarning]
+  -> Map OpaqueId OpaqueBlock
+  -> Map QName OpaqueId
   -> TCM ()
 addImportedThings isig metas ibuiltin patsyns display userwarn
-                  partialdefs warnings = do
+                  partialdefs warnings oblock oid = do
   stImports              `modifyTCLens` \ imp -> unionSignatures [imp, isig]
   stImportedMetaStore    `modifyTCLens` HMap.union metas
   stImportedBuiltins     `modifyTCLens` \ imp -> Map.union imp ibuiltin
@@ -269,6 +275,8 @@ addImportedThings isig metas ibuiltin patsyns display userwarn
   stPatternSynImports    `modifyTCLens` \ imp -> Map.union imp patsyns
   stImportedDisplayForms `modifyTCLens` \ imp -> HMap.unionWith (++) imp display
   stTCWarnings           `modifyTCLens` \ imp -> imp `List.union` warnings
+  stOpaqueBlocks         `modifyTCLens` \ imp -> imp `Map.union` oblock
+  stOpaqueIds            `modifyTCLens` \ imp -> imp `Map.union` oid
   addImportedInstances isig
 
 -- | Scope checks the given module. A proper version of the module
@@ -775,6 +783,8 @@ createInterfaceIsolated x file msrc = do
       display     <- useTC stImportsDisplayForms
       userwarn    <- useTC stImportedUserWarnings
       partialdefs <- useTC stImportedPartialDefs
+      opaqueblk   <- useTC stOpaqueBlocks
+      opaqueid    <- useTC stOpaqueIds
       ipatsyns <- getPatternSynImports
       ho       <- getInteractionOutputCallback
       -- Every interface is treated in isolation. Note: Some changes to
@@ -801,7 +811,7 @@ createInterfaceIsolated x file msrc = do
                stModuleToSource `setTCLens` mf
                setVisitedModules vs
                addImportedThings isig metas ibuiltin ipatsyns display
-                 userwarn partialdefs []
+                 userwarn partialdefs [] opaqueblk opaqueid
 
                r  <- createInterface x file NotMainInterface msrc
                mf' <- useTC stModuleToSource
@@ -838,9 +848,15 @@ chaseMsg
   -> TCM ()
 chaseMsg kind x file = do
   indentation <- (`replicate` ' ') <$> asksTC (pred . length . envImportPath)
+  traceImports <- optTraceImports <$> commandLineOptions
   let maybeFile = caseMaybe file "." $ \ f -> " (" ++ f ++ ")."
-      vLvl | kind == "Checking" = 1
-           | otherwise          = 2
+      vLvl | kind == "Checking"
+             && traceImports > 0 = 1
+           | kind == "Finished"
+             && traceImports > 1 = 1
+           | List.isPrefixOf "Loading" kind
+             && traceImports > 2 = 1
+           | otherwise = 2
   reportSLn "import.chase" vLvl $ concat
     [ indentation, kind, " ", prettyShow x, maybeFile ]
 
@@ -976,7 +992,7 @@ createInterface mname file isMain msrc = do
     stTokens `modifyTCLens` (fileTokenInfo <>)
 
     setOptionsFromSourcePragmas src
-    checkCohesionAttributes (srcCohesion src)
+    checkAttributes (srcAttributes src)
     syntactic <- optSyntacticEquality <$> pragmaOptions
     localTC (\env -> env { envSyntacticEqualityFuel = syntactic }) $ do
 
@@ -1174,7 +1190,7 @@ createInterface mname file isMain msrc = do
 -- 'MainInterface', the warnings definitely include also unsolved
 -- warnings.
 
-getAllWarnings' :: (MonadFail m, ReadTCState m, MonadWarning m) => MainInterface -> WhichWarnings -> m [TCWarning]
+getAllWarnings' :: (MonadFail m, ReadTCState m, MonadWarning m, MonadTCM m) => MainInterface -> WhichWarnings -> m [TCWarning]
 getAllWarnings' (MainInterface _) = getAllWarningsPreserving unsolvedWarnings
 getAllWarnings' NotMainInterface  = getAllWarningsPreserving Set.empty
 
@@ -1233,11 +1249,20 @@ buildInterface src topLevel = do
     optionsUsed <- useTC stPragmaOptions
     partialDefs <- useTC stLocalPartialDefs
 
+    -- Only serialise the opaque blocks actually defined in this
+    -- top-level module.
+    opaqueBlocks' <- useTC stOpaqueBlocks
+    opaqueIds' <- useTC stOpaqueIds
+    let
+      mh = moduleNameId (srcModuleName src)
+      opaqueBlocks = Map.filterWithKey (\(OpaqueId _ mod) _ -> mod == mh) opaqueBlocks'
+      opaqueIds    = Map.filterWithKey (\_ (OpaqueId _ mod) -> mod == mh) opaqueIds'
+
     -- Andreas, 2015-02-09 kill ranges in pattern synonyms before
     -- serialization to avoid error locations pointing to external files
     -- when expanding a pattern synonym.
     patsyns <- killRange <$> getPatternSyns
-    let builtin' = Map.mapWithKey (\ x b -> (x,) . primFunName <$> b) builtin
+    let builtin' = Map.mapWithKey (\ x b -> primName x <$> b) builtin
     warnings <- getAllWarnings AllWarnings
     let i = Interface
           { iSourceHash      = hashText source
@@ -1262,9 +1287,11 @@ buildInterface src topLevel = do
           , iPatternSyns     = patsyns
           , iWarnings        = warnings
           , iPartialDefs     = partialDefs
+          , iOpaqueBlocks    = opaqueBlocks
+          , iOpaqueNames     = opaqueIds
           }
     i <-
-      ifM (collapseDefault . optSaveMetas <$> pragmaOptions)
+      ifM (optSaveMetas <$> pragmaOptions)
         (return i)
         (do reportSLn "import.iface" 7
               "  instantiating all meta variables"
@@ -1275,6 +1302,10 @@ buildInterface src topLevel = do
     reportSLn "import.iface" 7 "  interface complete"
     return i
 
+    where
+      primName (PrimitiveName x) b = (x, primFunName b)
+      primName (BuiltinName x)   b = __IMPOSSIBLE__
+
 -- | Returns (iSourceHash, iFullHash)
 --   We do not need to check that the file exist because we only
 --   accept @InterfaceFile@ as an input and not arbitrary @AbsolutePath@!
