@@ -1,3 +1,5 @@
+{-# OPTIONS_GHC -Wunused-imports #-}
+
 {-# LANGUAGE NondecreasingIndentation #-}
 
 {-| Compile-time irrelevance.
@@ -73,12 +75,8 @@ variable rule:
 
 module Agda.TypeChecking.Irrelevance where
 
-import Control.Arrow (second)
-
 import Control.Monad.Except
 
-import qualified Data.Map as Map
-
 import Agda.Interaction.Options
 
 import Agda.Syntax.Common
@@ -90,185 +88,9 @@ import Agda.TypeChecking.Pretty
 import Agda.TypeChecking.Reduce
 import Agda.TypeChecking.Substitute.Class
 
-import Agda.Utils.Function
 import Agda.Utils.Lens
 import Agda.Utils.Maybe
 import Agda.Utils.Monad
-import Agda.Utils.WithDefault
-import qualified Agda.Utils.Null as Null
-import Agda.Utils.WithDefault (collapseDefault)
-
--- | data 'Relevance'
---   see "Agda.Syntax.Common".
-
--- * Operations on 'Dom'.
-
--- | Prepare parts of a parameter telescope for abstraction in constructors
---   and projections.
-hideAndRelParams :: (LensHiding a, LensRelevance a) => a -> a
-hideAndRelParams = hideOrKeepInstance . mapRelevance nonStrictToIrr
-
--- * Operations on 'Context'.
-
--- | Modify the context whenever going from the l.h.s. (term side)
---   of the typing judgement to the r.h.s. (type side).
-workOnTypes :: (MonadTCEnv m, HasOptions m, MonadDebug m)
-            => m a -> m a
-workOnTypes cont = do
-  allowed <- optExperimentalIrrelevance <$> pragmaOptions
-  verboseBracket "tc.irr" 60 "workOnTypes" $ workOnTypes' allowed cont
-
--- | Internal workhorse, expects value of --experimental-irrelevance flag
---   as argument.
-workOnTypes' :: (MonadTCEnv m) => Bool -> m a -> m a
-workOnTypes' experimental
-  = (if experimental
-     then modifyContextInfo (mapRelevance irrToNonStrict)
-     else id)
-  . applyQuantityToContext zeroQuantity
-  . typeLevelReductions
-  . localTC (\ e -> e { envWorkingOnTypes = True })
-
--- | (Conditionally) wake up irrelevant variables and make them relevant.
---   For instance,
---   in an irrelevant function argument otherwise irrelevant variables
---   may be used, so they are awoken before type checking the argument.
---
---   Also allow the use of irrelevant definitions.
-applyRelevanceToContext :: (MonadTCEnv tcm, LensRelevance r) => r -> tcm a -> tcm a
-applyRelevanceToContext thing =
-  case getRelevance thing of
-    Relevant -> id
-    rel      -> applyRelevanceToContextOnly   rel
-              . applyRelevanceToJudgementOnly rel
-
--- | (Conditionally) wake up irrelevant variables and make them relevant.
---   For instance,
---   in an irrelevant function argument otherwise irrelevant variables
---   may be used, so they are awoken before type checking the argument.
---
---   Precondition: @Relevance /= Relevant@
-applyRelevanceToContextOnly :: (MonadTCEnv tcm) => Relevance -> tcm a -> tcm a
-applyRelevanceToContextOnly rel = localTC
-  $ over eContext     (map $ inverseApplyRelevance rel)
-  . over eLetBindings (Map.map . fmap . second $ inverseApplyRelevance rel)
-
--- | Apply relevance @rel@ the the relevance annotation of the (typing/equality)
---   judgement.  This is part of the work done when going into a @rel@-context.
---
---   Precondition: @Relevance /= Relevant@
-applyRelevanceToJudgementOnly :: (MonadTCEnv tcm) => Relevance -> tcm a -> tcm a
-applyRelevanceToJudgementOnly = localTC . over eRelevance . composeRelevance
-
--- | Like 'applyRelevanceToContext', but only act on context if
---   @--irrelevant-projections@.
---   See issue #2170.
-applyRelevanceToContextFunBody :: (MonadTCM tcm, LensRelevance r) => r -> tcm a -> tcm a
-applyRelevanceToContextFunBody thing cont =
-  case getRelevance thing of
-    Relevant -> cont
-    rel -> applyWhenM (optIrrelevantProjections <$> pragmaOptions)
-      (applyRelevanceToContextOnly rel) $    -- enable local irr. defs only when option
-      applyRelevanceToJudgementOnly rel cont -- enable global irr. defs alway
-
--- | Sets the current quantity (unless the given quantity is 1).
-applyQuantityToContext :: (MonadTCEnv tcm, LensQuantity q) => q -> tcm a -> tcm a
-applyQuantityToContext thing =
-  case getQuantity thing of
-    Quantity1{} -> id
-    q           -> applyQuantityToJudgementOnly q
-
--- | Apply quantity @q@ the the quantity annotation of the (typing/equality)
---   judgement.  This is part of the work done when going into a @q@-context.
---
---   Precondition: @Quantity /= Quantity1@
-applyQuantityToJudgementOnly :: (MonadTCEnv tcm) => Quantity -> tcm a -> tcm a
-applyQuantityToJudgementOnly = localTC . over eQuantity . composeQuantity
-
--- | Apply inverse composition with the given cohesion to the typing context.
-applyCohesionToContext :: (MonadTCEnv tcm, LensCohesion m) => m -> tcm a -> tcm a
-applyCohesionToContext thing =
-  case getCohesion thing of
-    m | m == unitCohesion -> id
-      | otherwise         -> applyCohesionToContextOnly   m
-                             -- Cohesion does not apply to the judgment.
-
-applyCohesionToContextOnly :: (MonadTCEnv tcm) => Cohesion -> tcm a -> tcm a
-applyCohesionToContextOnly q = localTC
-  $ over eContext     (map $ inverseApplyCohesion q)
-  . over eLetBindings (Map.map . fmap . second $ inverseApplyCohesion q)
-
--- | Can we split on arguments of the given cohesion?
-splittableCohesion :: (HasOptions m, LensCohesion a) => a -> m Bool
-splittableCohesion a = do
-  let c = getCohesion a
-  pure (usableCohesion c) `and2M` (pure (c /= Flat) `or2M` do collapseDefault . optFlatSplit <$> pragmaOptions)
-
--- | (Conditionally) wake up irrelevant variables and make them relevant.
---   For instance,
---   in an irrelevant function argument otherwise irrelevant variables
---   may be used, so they are awoken before type checking the argument.
---
---   Also allow the use of irrelevant definitions.
---
---   This function might also do something for other modalities.
-applyModalityToContext :: (MonadTCEnv tcm, LensModality m) => m -> tcm a -> tcm a
-applyModalityToContext thing =
-  case getModality thing of
-    m | m == unitModality -> id
-      | otherwise         -> applyModalityToContextOnly   m
-                           . applyModalityToJudgementOnly m
-
--- | (Conditionally) wake up irrelevant variables and make them relevant.
---   For instance,
---   in an irrelevant function argument otherwise irrelevant variables
---   may be used, so they are awoken before type checking the
---   argument.
---
---   This function might also do something for other modalities, but
---   not for quantities.
---
---   Precondition: @Modality /= Relevant@
-applyModalityToContextOnly :: (MonadTCEnv tcm) => Modality -> tcm a -> tcm a
-applyModalityToContextOnly m = localTC
-  $ over eContext (map $ inverseApplyModalityButNotQuantity m)
-  . over eLetBindings
-      (Map.map . fmap . second $ inverseApplyModalityButNotQuantity m)
-
--- | Apply modality @m@ the the modality annotation of the (typing/equality)
---   judgement.  This is part of the work done when going into a @m@-context.
---
---   Precondition: @Modality /= Relevant@
-applyModalityToJudgementOnly :: (MonadTCEnv tcm) => Modality -> tcm a -> tcm a
-applyModalityToJudgementOnly = localTC . over eModality . composeModality
-
--- | Like 'applyModalityToContext', but only act on context (for Relevance) if
---   @--irrelevant-projections@.
---   See issue #2170.
-applyModalityToContextFunBody :: (MonadTCM tcm, LensModality r) => r -> tcm a -> tcm a
-applyModalityToContextFunBody thing cont = do
-    ifM (optIrrelevantProjections <$> pragmaOptions)
-      {-then-} (applyModalityToContext m cont)                -- enable global irr. defs always
-      {-else-} (applyRelevanceToContextFunBody (getRelevance m)
-               $ applyCohesionToContext (getCohesion m)
-               $ applyQuantityToContext (getQuantity m) cont) -- enable local irr. defs only when option
-  where
-    m = getModality thing
-
--- | Wake up irrelevant variables and make them relevant. This is used
---   when type checking terms in a hole, in which case you want to be able to
---   (for instance) infer the type of an irrelevant variable. In the course
---   of type checking an irrelevant function argument 'applyRelevanceToContext'
---   is used instead, which also sets the context relevance to 'Irrelevant'.
---   This is not the right thing to do when type checking interactively in a
---   hole since it also marks all metas created during type checking as
---   irrelevant (issue #2568).
---
---   Also set the current quantity to 0.
-wakeIrrelevantVars :: (MonadTCEnv tcm) => tcm a -> tcm a
-wakeIrrelevantVars
-  = applyRelevanceToContextOnly Irrelevant
-  . applyQuantityToJudgementOnly zeroQuantity
 
 -- | Check whether something can be used in a position of the given relevance.
 --
@@ -319,12 +141,11 @@ instance UsableRelevance a => UsableRelevance (Type' a) where
 
 instance UsableRelevance Sort where
   usableRel rel = \case
-    Type l -> usableRel rel l
-    Prop l -> usableRel rel l
-    Inf f n -> return True
-    SSet l -> usableRel rel l
+    Univ _ l -> usableRel rel l
+    Inf _ _ -> return True
     SizeUniv -> return True
     LockUniv -> return True
+    LevelUniv -> return True
     IntervalUniv -> return True
     PiSort a s1 s2 -> usableRel rel (a,s1,s2)
     FunSort s1 s2 -> usableRel rel (s1,s2)
@@ -397,7 +218,9 @@ instance UsableModality Term where
       return ok `and2M` usableMod mod vs
     Def f vs -> do
       fmod <- modalityOfConst f
-      let ok = fmod `moreUsableModality` mod
+      -- Pure modalities don't matter here, only positional ones, hence remove
+      -- them from the equation.
+      let ok = setCohesion Flat fmod `moreUsableModality` mod
       reportSDoc "tc.irr" 50 $
         "Definition" <+> prettyTCM (Def f []) <+>
         text ("has modality " ++ show fmod ++ ", which is a " ++
@@ -446,15 +269,6 @@ instance UsableRelevance a => UsableModality (Type' a) where
 
 instance UsableModality Sort where
   usableMod mod s = usableRel (getRelevance mod) s
-  -- usableMod mod s = case s of
-  --   Type l -> usableMod mod l
-  --   Prop l -> usableMod mod l
-  --   Inf    -> return True
-  --   SizeUniv -> return True
-  --   PiSort a s -> usableMod mod (a,s)
-  --   UnivSort s -> usableMod mod s
-  --   MetaS x es -> usableMod mod es
-  --   DummyS{} -> return True
 
 instance UsableModality Level where
   usableMod mod (Max _ ls) = usableRel (getRelevance mod) ls
@@ -500,7 +314,7 @@ usableAtModality' ms why mod t =
         Left blocker -> patternViolation blocker
   where
     formatWhy = do
-      compatible <- collapseDefault . optCubicalCompatible <$> pragmaOptions
+      compatible <- optCubicalCompatible <$> pragmaOptions
       cubical <- isJust . optCubical <$> pragmaOptions
       let
         context
@@ -580,12 +394,11 @@ isFibrant
   :: (LensSort a, PureTCM m, MonadBlock m)
   => a -> m Bool
 isFibrant a = abortIfBlocked (getSort a) <&> \case
-  Type{}     -> True
-  Prop{}     -> True
-  Inf f _    -> f == IsFibrant
-  SSet{}     -> False
+  Univ u _   -> univFibrancy u == IsFibrant
+  Inf u _    -> univFibrancy u == IsFibrant
   SizeUniv{} -> False
   LockUniv{} -> False
+  LevelUniv{}  -> False
   IntervalUniv{} -> False
   PiSort{}   -> False
   FunSort{}  -> False
@@ -599,12 +412,11 @@ isFibrant a = abortIfBlocked (getSort a) <&> \case
 --   pi type. (Notion by C. Sattler).
 isCoFibrantSort :: (LensSort a, PureTCM m, MonadBlock m) => a -> m Bool
 isCoFibrantSort a = abortIfBlocked (getSort a) <&> \case
-  Type{}     -> True
-  Prop{}     -> True
-  Inf f _    -> f == IsFibrant
-  SSet{}     -> False
+  Univ u _   -> univFibrancy u == IsFibrant
+  Inf u _    -> univFibrancy u == IsFibrant
   SizeUniv{} -> False
   LockUniv{} -> True
+  LevelUniv{}  -> False
   IntervalUniv{} -> True
   PiSort{}   -> False
   FunSort{}  -> False
